/************************************************
*
*	cmd_burn_optee.c
*
************************************************/
#include <command.h>
#include <common.h>
#include <asm/arch/system.h>

#include <asm/arch/sys_flash_layout.h>

/**********************************************
*
*	define
*
***********************************************/
#define PACK_INFO_HEADER_MAGIC	0x6330beef
#define EMMC_BLOCK_SIZE			512

#if defined(SYS_PARAM_RESERVED_START) && defined(SYS_PARAM_RESERVED_SIZE)
#define RESERVED_AREA_OFFSET		SYS_PARAM_RESERVED_START
#define RESERVED_AREA_SIZE			SYS_PARAM_RESERVED_SIZE
#else
#define RESERVED_AREA_SIZE			0x400000        // 4M
#define RESERVED_AREA_OFFSET		(SYS_PARAM_BOOTCODE_SIZE - RESERVED_AREA_SIZE)
#endif
/**********************************************
*
*	struct
*
***********************************************/
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
typedef struct key_info_t
{
	unsigned int		start_addr;		// 0~3
	unsigned int		end_addr;		// 4~7
	unsigned char		delete_enable;	// 8
	unsigned char		burn_enable;	// 9
	unsigned char		reserved[6];		// 10~15
	char				key_name[16];	// 16~31
	unsigned char		uuid[16]; 		// 32~47
}key_info;

typedef struct pack_info_t
{
	unsigned int magic;		// 0~3
	unsigned int key_cnt;		// 4~7
	unsigned int xor_value;	// 8~11
	unsigned int checksum;	// 12~15
	key_info	key[20];			// 16~975
	unsigned int reserved[12];	// 976~1023
}pack_info;


/*
*	struct info sync from https://jira.realtek.com/browse/ANDROIDTV-754
*/
typedef struct _keyinfo_start{
	unsigned char	uuid[16];
	unsigned int	keyLen;
	unsigned int	paddingLen;
	unsigned char	padding[8];	
}keyinfo_start;
 
typedef struct _keyinfo_end{
	unsigned char	shainfo[32];
	char			key_name[16];
	char			version[4];
	char			reserved[8];
	unsigned int	crc32; 
}keyinfo_end;


static unsigned int cal_checksum( unsigned int start_address, unsigned int len )
{
	unsigned int i, checksum = 0;
	unsigned char * pcBuf = (unsigned char *) start_address;
	for( i=0; i<len; i++ )
	{
		checksum += pcBuf[i];
	}
	return checksum;
}

static unsigned int align_to_boundary(unsigned int size, unsigned int block_size)
{
	return ((size % block_size) ? (size / block_size + 1)*block_size : (size / block_size)*block_size);
}


static int secureKey_info_get(void *data_buffer, unsigned char *uuid, char *key_name)
{
	keyinfo_start key_head;
	keyinfo_end key_tail;
	
	if(data_buffer == NULL || (uuid == NULL && key_name == NULL)){
		printf("[%s][%d] parameter set error!\r\n", __func__, __LINE__);
		return -1;
	}

	memcpy(&key_head, data_buffer, sizeof(keyinfo_start));

	if(uuid)
		memcpy(uuid, key_head.uuid, sizeof(key_head.uuid));

	if(key_name){
		memcpy(&key_tail, data_buffer + sizeof(keyinfo_start) + key_head.keyLen + key_head.paddingLen, sizeof(keyinfo_end));

		if(key_tail.version[0] == 'V' && key_tail.version[1] == 'E' && key_tail.version[2] == 'R')
			memcpy(key_name, key_tail.key_name, sizeof(key_tail.key_name));
		else
			snprintf(key_name, strlen(key_name), "%s", "Null");
	}

	return 0;
}

static int secureKey_clean_dirty_header_info(pack_info *pheader)
{
	unsigned int null_hit = 0;
	key_info	key_list[20];
	int i = 0, j=0;

	if(pheader == NULL){
		printf("[%s][%d] parameter error!\r\n", __func__, __LINE__);
		return -1;
	}

	for(i = 0; i < pheader->key_cnt; i++){
		if((!strcmp(pheader->key[i].key_name, "NULL") || !strcmp(pheader->key[i].key_name, "null") || !strcmp(pheader->key[i].key_name, "Null")) && \
			(pheader->key[i].burn_enable == 0)){
			null_hit++;
			continue;
		}else{
			memcpy((void *)(&key_list[j]), (void *)(&pheader->key[i]), sizeof(key_info));
			j++;
		}
	}

	if(null_hit){
		memset(pheader->key, 0x00, sizeof(pheader->key));
	
		for(i=0; i < j; i++)
			memcpy((void *)(&pheader->key[i]), (void *)(&key_list[i]), sizeof(key_info));

		pheader->key_cnt = j;
		pheader->xor_value = pheader->magic^pheader->key_cnt;
		pheader->checksum = 0;
		
		// checksum
		pheader->checksum = cal_checksum((unsigned int)pheader, sizeof(pack_info)); //[CID-314046]: wrong sizeof argument
		
		printf("Clean %d garbage key info!\r\n", null_hit);
	}

	return 0;
}

/**********************************************
*
*	return value:
*			-1	---->	load header fail 
*			0	---->	no header or no key record
*			1	---->	key record at reserved area
*
**********************************************/
static int secureKey_header_load_and_chk(pack_info *pheader)
{
	unsigned int blk_no = 0;
	unsigned int size =0;

	unsigned int check_sum = 0;
	unsigned int no_key = 0;

	if(pheader == NULL){
		printf("[%s][%d] header buffer is NULL\r\n", __func__, __LINE__);
		return -1;
	}

	if(RESERVED_AREA_OFFSET%EMMC_BLOCK_SIZE){
		printf("[%s][%d]RESERVED_AREA_OFFSET is not 512 align!\n", __func__, __LINE__);
		return -1;
	}

	if(sizeof(pack_info)!= 1024){
		printf("[%s][%d]header struct size is not 1024bytes!\n", __func__, __LINE__);
		return -1;
	}
	
	blk_no = RESERVED_AREA_OFFSET/EMMC_BLOCK_SIZE;
	size = sizeof(pack_info);
	if(!rtk_eMMC_read(blk_no, size, (unsigned int *)pheader)){
		printf("[%s][%d]read header data from emmc fail!\n", __func__, __LINE__);
		return -1;
	}

	// check header
	if(pheader->magic != PACK_INFO_HEADER_MAGIC){
		printf("[%s][%d] No header at reserverd area!\n",__func__, __LINE__);
		no_key = 1;
	}else{
		check_sum = cal_checksum((unsigned int)pheader, 3*sizeof(unsigned int));
		check_sum += cal_checksum((unsigned int)pheader + 4*sizeof(unsigned int), sizeof(pack_info)-4*sizeof(unsigned int));

		if(check_sum != pheader->checksum){
			printf("[%s][%d] header checksum error!\n",__func__, __LINE__);
			no_key = 1;
		}
	}

	if(no_key)
		return 0;
	else
		return 1;
}

static int secureKey_record_search(pack_info *pheader, char *key_name)
{
	int i = 0;
	int key_found = 0;
	int key_index = 0;
	
	if(pheader == NULL || key_name == NULL){
		printf("[%s][%d] parameter error!\r\n", __func__, __LINE__);
		return -1;
	}

	// search repeat secureKey
	for(i = 0; i < pheader->key_cnt; i++){
		if(!strcmp(pheader->key[i].key_name, "NULL") || \
			!strcmp(pheader->key[i].key_name, "null") || \
			!strcmp(pheader->key[i].key_name, "Null")){
			continue;
		}

		if(!strcmp(pheader->key[i].key_name, key_name)){
			printf("[%s][%d]Found key record, key_name %s \r\n", __func__, __LINE__, key_name);
			key_found = 1;
			key_index = i;
			break;
		}
	}

	if(key_found)
		return key_index;
	else
		return -1;
}

static int secureKey_repeatKey_handler(pack_info *pheader, char *key_name)
{
	int repeat_key_index = 0;

	if(pheader == NULL || key_name == NULL){
		printf("[%s][%d]parameter error!\r\n", __func__, __LINE__);
		return -1;
	}

	// search repeat secureKey
	repeat_key_index = secureKey_record_search(pheader, key_name);
	if(repeat_key_index < 0){
		printf("[%s][%d]No repeat key record!\r\n", __func__, __LINE__);
		return 0;
	}

	// delete repeat key data 
	if(pheader->key[repeat_key_index].burn_enable){
		unsigned int data_offset = 0;
		unsigned int blk_no = 0;
		unsigned int data_size = 0;
		unsigned int tmp_addr = TEMP_BUFFER_FOR_FLASH_DATA_ADDR+0x200000;	// don't care buffer context

		data_offset = pheader->key[repeat_key_index].start_addr + RESERVED_AREA_OFFSET;
		if(data_offset % EMMC_BLOCK_SIZE){
			printf("[%s][%d] key data offset - 0x%x is not align!but we don't care it!\r\n", __func__, __LINE__, data_offset);
			goto update_header;
		}

		blk_no = data_offset/EMMC_BLOCK_SIZE;
		data_size = pheader->key[repeat_key_index].end_addr- pheader->key[repeat_key_index].start_addr;

		if((data_offset + data_size) >= SYS_PARAM_BOOTCODE_SIZE){
			printf("[%s][%d] key data beyond bootcode area boundary!but we don't care it![0x%x]\r\n", \
				__func__, __LINE__, data_offset + data_size);
			goto update_header;
		}
		
		// use tmp buffer delete key data		
		if(!rtk_eMMC_write(blk_no, data_size, (unsigned int *)tmp_addr)){
			printf("[%s][%d]delete data fail!\n", __func__, __LINE__);
			goto update_header;
		}
	}

update_header:
	// update header
	if(repeat_key_index == (pheader->key_cnt-1)){
		memset(&(pheader->key[repeat_key_index]), 0x00, sizeof(key_info));
	}else{
		int i = 0;
		for(i=repeat_key_index; i<(pheader->key_cnt-1); i++){
			memcpy(&(pheader->key[i]), &(pheader->key[i+1]), sizeof(key_info));
		}
		memset(&(pheader->key[pheader->key_cnt-1]), 0x00, sizeof(key_info));
	}
	
	pheader->key_cnt--;

	return 0;
}
#endif

int burn_secureKey(unsigned int key_addr, unsigned int key_size, char * key_name)
{
	int ret = -1;
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	pack_info header_info;
	unsigned int no_key = 0;

	unsigned int key_cnt = 0;
	unsigned int start_addr = 0;
	unsigned int end_addr =0;

	unsigned int blk_no = 0;
	unsigned int size = 0;
	
	unsigned char uuid_array[16];
	char keyname_array[16];

	if((key_addr == 0) || (key_size == 0)){
		printf("[%s][%d]Parameter error!\n", __func__, __LINE__);
		return ret;
	}

	memset(&header_info, 0x00, sizeof(header_info));

	// loader secureKey header
	ret = secureKey_header_load_and_chk(&header_info);
	if(ret < 0){
		printf("[%s][%d]load secureKey header fail!\r\n",__func__, __LINE__);
		return -1;
	}

	// key status
	no_key = (ret == 0) ? 1: 0;

	memset(&uuid_array, 0x00, sizeof(uuid_array));
	memset(&keyname_array, 0x00, sizeof(keyname_array));

	// init key_name and uuid information
	if(key_name != NULL){
		memcpy(keyname_array, key_name, strlen(key_name));
	}else{
		ret = secureKey_info_get((void *)key_addr, uuid_array, keyname_array);
		if(ret < 0){
			printf("[%s][%d] get secureKey info error!\r\n", __func__, __LINE__);
			return -1;
		}
	}  

	if(no_key){
		memset(&header_info, 0x00, sizeof(header_info));
		key_cnt = 0;
		start_addr = RESERVED_AREA_OFFSET + sizeof(header_info);
		end_addr = start_addr + key_size;
	}else{
		ret = secureKey_clean_dirty_header_info(&header_info);
		if(ret < 0){
			printf("[%s][%d] clean dirth record info error!\r\n", __func__, __LINE__);
			return -1;
		}
		
		ret = secureKey_repeatKey_handler(&header_info, keyname_array);
		if(ret < 0){
			printf("[%s][%d] handle repeat key error!\r\n", __func__, __LINE__);
			return -1;
		}
		
		header_info.checksum = 0;	// clean checksum
		key_cnt = header_info.key_cnt;

		if(key_cnt == 0){
			start_addr = RESERVED_AREA_OFFSET + sizeof(header_info);
		}else{
			if(header_info.key[key_cnt - 1].burn_enable == 0){
				int i = 0;
				int hit_flag = 0;
				for(i = (key_cnt-1); i >= 0; i--){
					if(header_info.key[i].burn_enable == 1){
						hit_flag = 1;
						start_addr = align_to_boundary((header_info.key[i].end_addr+RESERVED_AREA_OFFSET), EMMC_BLOCK_SIZE);
						break;
					}
				}

				if(!hit_flag)
					start_addr = RESERVED_AREA_OFFSET + sizeof(header_info);
			}else{
				start_addr= align_to_boundary((header_info.key[key_cnt - 1].end_addr+RESERVED_AREA_OFFSET), EMMC_BLOCK_SIZE);
			}
		}
		
		end_addr = start_addr + key_size;
	}

	if(end_addr > (RESERVED_AREA_OFFSET + RESERVED_AREA_SIZE)){
		printf("[%s][%d] data too long...\n ", __func__, __LINE__);
		return -1;
	}
	
	// write data to flash
	blk_no = start_addr/EMMC_BLOCK_SIZE;
	size = key_size;
	if(!rtk_eMMC_write(blk_no, size, (unsigned int *)key_addr)){
		printf("[%s][%d]write data to emmc fail!\n", __func__, __LINE__);
		return ret;
	}

	// update header
	header_info.magic = PACK_INFO_HEADER_MAGIC;
	header_info.key_cnt = key_cnt +1;
	header_info.xor_value = header_info.magic^header_info.key_cnt;
	header_info.key[key_cnt].start_addr = start_addr - RESERVED_AREA_OFFSET;
	header_info.key[key_cnt].end_addr= end_addr - RESERVED_AREA_OFFSET;
	header_info.key[key_cnt].burn_enable = 1;
	header_info.key[key_cnt].delete_enable = 0;
	snprintf(header_info.key[key_cnt].key_name, sizeof(header_info.key[key_cnt].key_name), "%s", keyname_array);
	memcpy(header_info.key[key_cnt].uuid, uuid_array, sizeof(uuid_array));
	
	header_info.checksum = cal_checksum((unsigned int)&header_info, sizeof(header_info));

	// write header to flash
	blk_no = RESERVED_AREA_OFFSET/EMMC_BLOCK_SIZE;
	size = sizeof(header_info);
	if(!rtk_eMMC_write(blk_no, size, (unsigned int *)&header_info)){
		printf("[%s][%d]write data to emmc fail!\n", __func__, __LINE__);
		return ret;
	}

	return 0;
#else
	printf("[%s][%d]Not support this flash type!\n", __func__,__LINE__);
	return ret;
#endif
}

int list_secureKey(char *key_name)
{
	int ret = -1;
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	pack_info header_info;

	ret = secureKey_header_load_and_chk(&header_info);
	if(ret <= 0){
		if(ret < 0){
			printf("[%s][%d]read secureKey header fail!\r\n",__func__, __LINE__);
			return -1;
		}else{
			printf("[%s][%d]No secureKey header at reserved area!\r\n",__func__, __LINE__);
			return -1;
		}
	}

	ret = secureKey_record_search(&header_info, key_name);
	if(ret < 0){
		printf("[%s][%d]No secureKey %s record!\r\n",__func__, __LINE__, key_name);
		return -1;
	}

	return 0;	
#else
	printf("[%s][%d]Not support this flash type!\n", __func__,__LINE__);
	return ret;
#endif
}

int delete_secureKey(char *key_name)
{
	int ret = -1;
#if defined(CONFIG_SYS_RTK_EMMC_FLASH)
	pack_info header_info;
	unsigned int blk_no = 0;
	unsigned int size = 0;

	ret = secureKey_header_load_and_chk(&header_info);
	if(ret <= 0){
		if(ret < 0){
			printf("[%s][%d]read secureKey header fail!\r\n",__func__, __LINE__);
			return -1;
		}else{
			printf("[%s][%d]No secureKey header at reserved area!\r\n",__func__, __LINE__);
			return -1;
		}
	}

	ret = secureKey_record_search(&header_info, key_name);
	if(ret < 0){
		printf("[%s][%d]No secureKey %s record!\r\n",__func__, __LINE__, key_name);
		return -1;
	}

	// set key delete_enable
	header_info.key[ret].delete_enable = 1;
	if(header_info.key[ret].burn_enable == 1){
		header_info.key[ret].burn_enable = 0;
		header_info.key[ret].start_addr = 0;
		header_info.key[ret].end_addr = 0;
	}
	
	// caculute check sum
	header_info.checksum = 0;	// clean checksum
	header_info.checksum = cal_checksum((unsigned int)&header_info, sizeof(header_info));

	// write header to flash
	blk_no = RESERVED_AREA_OFFSET/EMMC_BLOCK_SIZE;
	size = sizeof(header_info);
	if(!rtk_eMMC_write(blk_no, size, (unsigned int *)&header_info)){
		printf("[%s][%d]write secureKey header data to emmc fail!\n", __func__, __LINE__);
		return -1;
	}
	
	return 0;
#else
	printf("[%s][%d]Not support this flash type!\n", __func__,__LINE__);
	return ret;
#endif

}

//#define _DEBUG_BURN_KEY_FLOW
#ifdef _DEBUG_BURN_KEY_FLOW
int do_secureKey_header_parse (cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = 0;
	pack_info header_info;
	unsigned int i = 0;

	if(argc != 2 || strcmp(argv[1], "parse") != 0)
		return CMD_RET_USAGE;

	ret = secureKey_header_load_and_chk(&header_info);
	if(ret <= 0){
		if(ret < 0){
			printf("[%s][%d]read secureKey header fail!\r\n",__func__, __LINE__);
			return -1;
		}else{
			printf("[%s][%d]No secureKey header at reserved area!\r\n",__func__, __LINE__);
			return -1;
		}
	}

	printf("\r\n=============================================\r\n");
	printf("Header magic	: 0x%x \r\n", header_info.magic);
	printf("Record keycnt	: 0x%x \r\n", header_info.key_cnt);
	printf("Xor value		: 0x%x \r\n", header_info.xor_value);
	printf("Checksum		: 0x%x \r\n", header_info.checksum);
	printf("\r\n=============================================\r\n\n");
	
	for(i = 0; i<header_info.key_cnt; i++){
		printf("\r\n*********************************************\r\n");
		printf("Key-%d Info	:\r\n", i);
		printf("Start addr	: 0x%x \r\n", header_info.key[i].start_addr ? 
			(header_info.key[i].start_addr + RESERVED_AREA_OFFSET): header_info.key[i].start_addr);
		printf("End addr	: 0x%x \r\n", header_info.key[i].end_addr ?
			(header_info.key[i].end_addr + RESERVED_AREA_OFFSET): header_info.key[i].end_addr);
		printf("delete flag	: 0x%x \r\n", header_info.key[i].delete_enable);
		printf("burn flag	: 0x%x \r\n", header_info.key[i].burn_enable);
		printf("key name	: %s \r\n", header_info.key[i].key_name);
		printf("key uuid	: %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x %x \r\n", 
			header_info.key[i].uuid[0], header_info.key[i].uuid[1], header_info.key[i].uuid[2], header_info.key[i].uuid[3],
			header_info.key[i].uuid[4], header_info.key[i].uuid[5], header_info.key[i].uuid[6], header_info.key[i].uuid[7],
			header_info.key[i].uuid[8], header_info.key[i].uuid[9], header_info.key[i].uuid[10], header_info.key[i].uuid[11],
			header_info.key[i].uuid[12], header_info.key[i].uuid[13], header_info.key[i].uuid[14], header_info.key[i].uuid[15]);
		printf("\r\n*********************************************\r\n");
	}

	return CMD_RET_SUCCESS;
}


U_BOOT_CMD
(
	secureKey, 3, 1,   do_secureKey_header_parse, //do not display usage,avoid customer abuse
	NULL/*"burn image to factory via uart by rtice"*/,
	NULL/*"argu\n"
	"secureKey parse\n"*/
);
#endif
