;;----------------------------------------------------------------------------
;;
;;The confidential and proprietary information contained in this file may
;;only be used by a person authorised under and to the extent permitted
;;by a subsisting licensing agreement from ARM Limited.
;;
;;        (C) COPYRIGHT 2008-2009,2011,2013 ARM Limited.
;;             ALL RIGHTS RESERVED
;;             
;;This entire notice must be reproduced on all copies of this file
;;and copies of this file may only be made by a person if such person is
;;permitted to do so under the terms of a subsisting license agreement
;;from ARM Limited.
;;
;;    SVN Information
;;
;;    Checked In          : $Date$
;;
;;    Revision            : $Revision$
;;
;;    Release Information : r0p0-00rel0
;;
;;-----------------------------------------------------------------------------
;; 
;;-----------------------------------------------------------------------------
;; Abstract :Interrupt Handelling Routines
;;-----------------------------------------------------------------------------
;; Overview
;;----------
;; Initialise the GICs
;;
;; For A9    tested with external and internal gic
;; For A1176 tested with external gic
;;----------------------------------------------------------------------------

        PRESERVE8
        AREA    boot, CODE

        ENTRY

        ;-----------------------------------------------
        ; Functions exported:
        ;-----------------------------------------------
        EXPORT entrypoint
        EXPORT  __user_setup_stackheap
        
        ;-----------------------------------------------
        ; Functions used
        ;-----------------------------------------------
        IMPORT initialize_cpu
        IMPORT invalidate_dcache_raw
        IMPORT enable_caches_tlb
        IMPORT uart_init

        [ ENABLE_SMP
        ; If using multiple cores - use this as entry point for later processors
        IMPORT  main_secondary                
        ]
        
        ;---------------------------------------------
        ; Configure RVCT to use a two memory region system
        IMPORT __use_two_region_memory

        ;---------------------------------------------
        ; STACK SIZES
        ; We just setup IRQ + SVC stacks
        ; We allow a total of 16kb per CPU
        ; 8 KB on SVC stack; and then 8KB below that for IRQ stack
	IMPORT ||Image$$HEAP$$ZI$$Base||
	IMPORT ||Image$$STACK$$ZI$$Base||
	IMPORT ||Image$$STACK$$ZI$$Limit||
        
STACK_BITS_PER_CPU EQU 14      ; allow 16kB per CPU - 8KB SVC ; 8KB IRQ
SVC_STACK_SIZE     EQU 1 << 13 ; SVC stack size
                   
        ;---------------------------------------------
        ; Peripheral base register
        [ :DEF: PL310_BASE
        |
                        GBLA    PL310_BASE
PL310_BASE              SETA    0x1f002000
        ]
	
entrypoint
        ; Read the architecture to see if ARMv6
        ; set vbar to vector_base address if v7 and above.
		; Helps to place vectors at any address
        [ ARMv6
        mov    r0,#0
        |
		mov		r0, #START_ADDRESS
		mcr     p15, 0, r0, c12, c0, 0
        ]

        ; Read the architecture to see if ARMv6
        ; Then decide if lead_processor or a secondary_processor
        [ ARMv6
        mov    r0,#0
        |
        mrc    p15, 0, r0, c0, c0, 5
        ]
        ands   r0, r0, #0xf
        BNE    secondary_boot
                
lead_processor
        ;------------------------------------
        ; Step 1: 
        ; Call routine to initialise processor
        ; and leave in a state with the DCache invalidated and disabled                
        ;------------------------------------
        BL      initialize_cpu
        
        [ ENABLE_PL310
        ; Enable the l2 cache
        BL      pl310_start_invalidate
        ]

	BL      invalidate_dcache_raw

        ;------------------------------------
        ; Step 2:
        ; Setup stack pointers - SVC and IRQ
        ;------------------------------------
        BL      setup_stacks

        ;------------------------------------
        ; Step 3:
        ; initialise SCU if primary processor.
        ;------------------------------------

        ; Set up SCU
        [ ENABLE_SMP
        MRC p15,0,r1,c15,c4,0                    ; Read SCU base address
        
        ;   Invalidate all: To SCU Invalidate All register
        MOV     r0, #0xFF                       ; Invalidate all ways of CPU0 and CPU1
        ORR     r0, #0xFF << 8                  ; Invalidate all ways of CPU2 and CPU3
        STR     r0, [r1, #0xc]                  ; SCU invalidate all register

        ;   Enable SCU: write 1 to control register
        LDR     r0, [r1]                        ; SCU control register
        ORR     r0, r0, #1                      ; Enable SCU
        STR     r0, [r1]                        ; SCU control register

        ;   Send event - to wakeup secondary CPUs - they can now enable their caches
        SEV
        ]

        ;------------------------------------
        ; Step 4:
        ; Finish CPU initialisation.
        ; Enable caches; turn on TLBs
        ;------------------------------------
        BL      enable_caches_tlb
        BL      configure_cpu
        BL      uart_init
	
        [ ENABLE_PL310
        ; Enable the l2 cache
        BL      pl310_wait_invalidate_done_and_enable
        ]

        ;------------------------------------
        ; Step 5:
        ; C library init and run
        ;------------------------------------
        IMPORT  __main       ; import label to __main
        B       __main       ; branch to C Library entry

secondary_boot
        ; Only execute this bit if trying to enable SMP
        ; Otherwise we skip to the wait_for_ever loop
        [ ENABLE_SMP
        
        ;------------------------------------
        ; Step 1: 
        ; Call routine to initialise processor
        ; and leave in a state with the DCache invalidated and disabled                
        ;------------------------------------
        BL      initialize_cpu
	BL      invalidate_dcache_raw

        ;------------------------------------
        ; Step 2:
        ; Setup stack pointers - SVC and IRQ
        ;------------------------------------
        BL      setup_stacks
                
        ;------------------------------------
        ; Step 3:
        ;
        ; Now wait for primary processor to complete initialisation
        ; of system before completing initialisation.
        ;
        ;------------------------------------
        MRC p15,0,r1,c15,c4,0                    ; Read SCU base address
wait_scu_enable
        WFE                
        LDR     r0, [r1]                        ; SCU control register
        AND     r0, r0, #1                      ; Enable SCU
        CMP     r0, #1
        BNE     wait_scu_enable
        
        ;------------------------------------
        ; Step 4:
        ;
        ; Finish CPU initialisation
        ;
        ;------------------------------------
        BL      enable_caches_tlb
        BL      configure_cpu
        BL      main_secondary
        ]
               
wait_for_ever
        [ ARMv6                
        mcr     p15, 0, r0, c7, c0, 4
        |
        WFI
        ]
        B       wait_for_ever        
        
	
;---------------------------------------------------------------------------------------------
setup_stacks              
        [ ENABLE_SMP
        ; Set the stack based on the CPU                
        MRC     p15, 0, r0, c0, c0, 5
        AND     r0, r0, #0x3
        |
        ; If not SMP then we are processor 0
        MOV     r0, #0
        ]

        ; Setup SVC stack per CPU
        LDR     sp, =||Image$$STACK$$ZI$$Limit||
        ; Set up SVC stack pointer for each CPU
        SUB     r3, sp, r0, lsl #STACK_BITS_PER_CPU
        MOV     sp, r3
	
        ; Set up IRQ stack pointer for each CPU
        ; IRQ Stack starts directly below SVC stack
        MRS     r0, cpsr    ; save current state
        MOV     r2, #0xd2   ; ARM state, IRQ mode, I+F disabled
        MSR     cpsr_c, r2  ; enter IRQ mode
	SUB     r3,r3,#SVC_STACK_SIZE
        MOV     sp, r3

        ; Back to SVC mode
        MSR     cpsr_c, r0

        BX      lr

;---------------------------------------------------------------------------------------------                
configure_cpu
        ; Enable speculative Data Prefetching, and SMP
        ; Check if multi-processor
        ; Z flag will be set if bit30 of register is 0 [multiproc].

        ; If enabling SMP setup r1 with SMP flag
        MOV    r1,#0
        [ ENABLE_SMP
        mrc    p15, 0, r0, c0, c0, 5
        tst    r0, #1 << 30
        MOVEQ  r1,#1<<6                         ; SMP bit
        ORREQ r0, r0, #1 << 0                   ; Cache and TLB maintenance broadcat
        ]

        MRC p15, 0, r0, c1, c0, 1               ; Auxiliary control register
        ORR r0, r0, r1                          ; Set SMP flag as required (see above)
        ORR r0, r0, #1 << 2                     ; L1 prefetch enable (D side)
        ORR r0, r0, #1 << 1                     ; L2 prefetch enable
        MCR p15, 0, r0, c1, c0, 1               ; Auxiliary control register
	
        [ ARMv6
        MOV     r0, #0
        MCR     p15, 0, r0, c7, c5, 4  ; prefetch flush
        |
        ISB
        ]

        ; Enable VFP
        MRC      p15, 0, r0, c1, c0, 2  ; enable coprocessor access
        ORR      r0, r0, #(0xf << 20)   ; to both p10 & p11
        MCR      p15, 0, r0, c1, c0, 2  ; for VFP/Neon

        [ ENABLE_FP
        FMRX     r1, FPEXC
        ORR      r1, r1, #1<<30         ; FPEXC_ENABLE bit
        FMXR     FPEXC, r1

        FMRX     r1, FPSCR
        ORR      r1, r1, #3<<24         ; Flush to Zero && Default NaN
        FMXR     FPSCR, r1
        ]

        BX lr

;---------------------------------------------------------------------------------------------
; RVCT C Library initialisation:
;
; This is called by __rt_entry in the RVCT C Library
; It sets up stack limit and heap base/limit
; Stack pointers already setup by this point
;    
;---------------------------------------------------------------------------------------------
__user_setup_stackheap
        ; Return:
        ; r0 - heap base
        ; r2 - heap limit
        ; r3 - stack limit
        ; sp - stack base [unchanged by this function]

        ; Setup heap base + limit first
	LDR     r0, =||Image$$HEAP$$ZI$$Base||        ; Heap base
        LDR     r2, =||Image$$STACK$$ZI$$Base||       ; Heap limit is stack Base
        ; Heap limit is just below stacks [allow 4 cpus]

        ; Then set the stack limit
        ; First get the top of stack for this CPU
        ; And subtract STACKSIZE*(CPU Number) + SVC_STACK_SIZE
        
        ; r1 <- CPU_Num + 1
        [ ARMv6
        MOV     r1,#0
        |
        MRC     p15, 0, r1, c0, c0, 5
        ]
        AND     r1, r1, #0x3

        ; Top of stacks
        LDR     r3, =||Image$$STACK$$ZI$$Limit||
        
        ; Subtract CPUNumber+1 * STACKSIZE
        SUB     r3, r3, r1, lsl #STACK_BITS_PER_CPU
        SUB     r3, r3, #SVC_STACK_SIZE

        BX      lr

	
;------------------------------------------------------------------------------------
L2_CONTROL              EQU     0x100
L2_INV_WAY              EQU     0x77c
                        
pl310_start_invalidate
        LDR     r1, =PL310_BASE
	
        ;; Ensure PL310 disabled
        MOV     r2, #0
        STR     r2, [r1, #L2_CONTROL]
	
        ;; Invalidate the cache
        LDR     r2, =0xFFFF
        STR     r2, [r1, #L2_INV_WAY]
        BX      lr
                	
pl310_wait_invalidate_done_and_enable
        LDR     r1, =PL310_BASE
        LDR     r2, =0xFFFF
        ;; Wait for invalidate to be complete
inv_loop
        LDR     r3, [r1, #L2_INV_WAY]
        ANDS    r3, r3, r2
        BNE     inv_loop

        ;; Turn on cache
        MOV     r2, #1
        STR     r2, [r1, #L2_CONTROL]
        
        BX      lr
                                
;---------------------------------------------------------------------------------------------

        END
