/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited or its affiliates.
*
*        (C) COPYRIGHT 2008-2009, 2011-2017 ARM Limited or its affiliates.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited or its affiliates.
*
* Release   : r0p0-00eac0
*-----------------------------------------------------------------------------*/

/* 

BRIEF DESCRIPTION, PURPOSE AND STRATEGY
=======================================

This test checks that the interrupts are correctly connected by directly writing to their RAW status registers.


DEBUG HINTS IN CASE OF FAILURE
===========================
a) Check the AXI bus connections as this is the bus that is used to communicate with the Job manager.
b) Check that the GPU's reset signal is not asserted.
c) Check that the GPU input clock is toggling as expected.
d) Check that the DFT signals are disabled.
e) Check that MBIST is disabled.
f) Check the GPU interrupt signals are connected correctly i.e. IRQGPU, IRQMMU, IRQJOB and IRQEVENT.

*/

#include "MaliFns.h"


int RunMaliTest_interrupt_DVALIN_part0 (void);
int RunMaliTest_interrupt_DVALIN_part1 (void);
int RunMaliTest_interrupt_DVALIN_part2 (void);
int RunMaliTest_interrupt_DVALIN_part3 (void);
int RunMaliTest_interrupt_DVALIN_part4 (void);

static int mali_step;

int RunMaliTest_interrupt_DVALIN (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_interrupt_DVALIN_part0();
    case 1  : return RunMaliTest_interrupt_DVALIN_part1();
    case 2  : return RunMaliTest_interrupt_DVALIN_part2();
    case 3  : return RunMaliTest_interrupt_DVALIN_part3();
    case 4  : return RunMaliTest_interrupt_DVALIN_part4();
    default : {printf ("mali_step=%d", mali_step); return 0;};
  };
};

int RunMaliTest_interrupt_DVALIN_part0 (void) {
  int res = 0;

  int num_cores, i, core_bitmap;

  printf("RUNNING TEST: interrupt\n");
  printf("  Purpose: Check interrupt line connectivity\n");
  printf("  Will assert all four interrupt signals sequentially.\n");

  // Get current configuration to allow testing all registers
  core_bitmap   = Mali_RdReg(0x0100);

  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i &= i-1;
  }

  /* The configuration is now printed out by Mali_Identify()
  printf("Selected configuration:\n");
  printf("  Bus width: %d\n", axi_width);
  printf("  Number of shader cores: %d\n", num_cores);
  printf("  L2 cache size: %d kB\n", ((1<<l2_size)/1024));
  */
  
  // Clear all interrupt registers
  Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
  Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
  Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
  Mali_WrReg(0x1024,0xFFFFFFFF); // EVENT_IRQ_CLEAR 

  // Enable GPU interrupt source, disable all other interrupt sources
  Mali_WrReg(0x0028,0x00000001); // GPU_IRQ_MASK
  Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
  Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
  Mali_WrReg(0x1028,0x00000000); // EVENT_IRQ_MASK

  // Trigger GPU interrupt
  Mali_WrReg(0x0020,0x00000001); // GPU_IRQ_RAWSTAT

  mali_step++;
  if (res == 0) {
   return 255;
  }
  return res;
}

int RunMaliTest_interrupt_DVALIN_part1 (void) {
  int gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status;
  gpu_irq_status   = Mali_RdReg(0x002c);
  job_irq_status   = Mali_RdReg(0x100c);
  mmu_irq_status   = Mali_RdReg(0x200c);
  event_irq_status = Mali_RdReg(0x102c);
  if( (gpu_irq_status == 1) && (job_irq_status == 0) && (mmu_irq_status == 0) && (event_irq_status == 0) ) {
    printf("GPU Interrupt asserted\n");
    Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
    Mali_WrReg(0x0028,0x00000000); // GPU_IRQ_MASK
    Mali_WrReg(0x1008,0x00000001); // JOB_IRQ_MASK
    Mali_WrReg(0x1000,0x00000001); // JOB_IRQ_RAWSTAT
    Mali_WrReg(0x0020,0x00000000); // GPU_IRQ_RAWSTAT
    mali_step++;
    return 255;
  }
  printf("FAILURE: Expected GPU interrupt. Got the following interrupts: GPU: %x  JOB: %x  MMU: %x  EVENT: %x\n",
         gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status);
  Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
  Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
  Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
  Mali_WrReg(0x1024,0xFFFFFFFF); // EVENT_IRQ_CLEAR 
  Mali_WrReg(0x0028,0x00000000); // GPU_IRQ_MASK
  Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
  Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
  Mali_WrReg(0x1028,0x00000000); // EVENT_IRQ_MASK
  return 1;
}

int RunMaliTest_interrupt_DVALIN_part2 (void) {
  int gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status;
  gpu_irq_status   = Mali_RdReg(0x002c);
  job_irq_status   = Mali_RdReg(0x100c);
  mmu_irq_status   = Mali_RdReg(0x200c);
  event_irq_status = Mali_RdReg(0x102c);
  if( (gpu_irq_status == 0) && (job_irq_status == 1) && (mmu_irq_status == 0) && (event_irq_status == 0) ) {
    printf("JOB Interrupt asserted\n");
    Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
    Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
    Mali_WrReg(0x2008,0x00000001); // MMU_IRQ_MASK
    Mali_WrReg(0x2000,0x00000001); // MMU_IRQ_RAWSTAT
    Mali_WrReg(0x1000,0x00000000); // JOB_IRQ_RAWSTAT
    mali_step++;
    return 255;
  }
  printf("FAILURE: Expected JOB interrupt. Got the following interrupts: GPU: %x  JOB: %x  MMU: %x  EVENT: %x\n",
         gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status);
  Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
  Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
  Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
  Mali_WrReg(0x1024,0xFFFFFFFF); // EVENT_IRQ_CLEAR 
  Mali_WrReg(0x0028,0x00000000); // GPU_IRQ_MASK
  Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
  Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
  Mali_WrReg(0x1028,0x00000000); // EVENT_IRQ_MASK
  return 1;
}

int RunMaliTest_interrupt_DVALIN_part3 (void) {
  int gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status;
  gpu_irq_status = Mali_RdReg(0x002c);
  job_irq_status = Mali_RdReg(0x100c);
  mmu_irq_status = Mali_RdReg(0x200c);
  event_irq_status = Mali_RdReg(0x102c);
  if( (gpu_irq_status == 0) && (job_irq_status == 0) && (mmu_irq_status == 1) && (event_irq_status == 0) ) {
    printf("MMU Interrupt asserted\n");
    Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
    Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
    Mali_WrReg(0x1028,0x00000001); // EVENT_IRQ_MASK
    Mali_WrReg(0x1020,0x00000001); // EVENT_IRQ_RAWSTAT
    Mali_WrReg(0x2000,0x00000000); // MMU_IRQ_RAWSTAT
    mali_step++;
    return 255;
  }
  printf("FAILURE: Expected MMU interrupt. Got the following interrupts: GPU: %x  JOB: %x  MMU: %x  EVENT: %x\n",
         gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status);
  Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
  Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
  Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
  Mali_WrReg(0x1024,0xFFFFFFFF); // EVENT_IRQ_CLEAR 
  Mali_WrReg(0x0028,0x00000000); // GPU_IRQ_MASK
  Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
  Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
  Mali_WrReg(0x1028,0x00000000); // EVENT_IRQ_MASK
  return 1;
}

int RunMaliTest_interrupt_DVALIN_part4 (void) {
  int gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status;
  gpu_irq_status   = Mali_RdReg(0x002c);
  job_irq_status   = Mali_RdReg(0x100c);
  mmu_irq_status   = Mali_RdReg(0x200c);
  event_irq_status = Mali_RdReg(0x102c);
  if( (gpu_irq_status == 0) && (job_irq_status == 0) && (mmu_irq_status == 0) && (event_irq_status == 1) ) {
    printf("EVENT Interrupt asserted\n");
  Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
  Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
  Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
  Mali_WrReg(0x1024,0xFFFFFFFF); // EVENT_IRQ_CLEAR 
  Mali_WrReg(0x0028,0x00000000); // GPU_IRQ_MASK
  Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
  Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
  Mali_WrReg(0x1028,0x00000000); // EVENT_IRQ_MASK
    mali_step++;
    return 0;
  }
  printf("FAILURE: Expected EVENT interrupt. Got the following interrupts: GPU: %x  JOB: %x  MMU: %x  EVENT: %x\n",
         gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status);
  Mali_WrReg(0x0024,0xFFFFFFFF); // GPU_IRQ_CLEAR
  Mali_WrReg(0x1004,0xFFFFFFFF); // JOB_IRQ_CLEAR
  Mali_WrReg(0x2004,0xFFFFFFFF); // MMU_IRQ_CLEAR
  Mali_WrReg(0x1024,0xFFFFFFFF); // EVENT_IRQ_CLEAR 
  Mali_WrReg(0x0028,0x00000000); // GPU_IRQ_MASK
  Mali_WrReg(0x1008,0x00000000); // JOB_IRQ_MASK
  Mali_WrReg(0x2008,0x00000000); // MMU_IRQ_MASK
  Mali_WrReg(0x1028,0x00000000); // EVENT_IRQ_MASK
  return 1;
}
