/*----------------------------------------------------------------------------
 *
 * The confidential and proprietary information contained in this file may
 * only be used by a person authorised under and to the extent permitted
 * by a subsisting licensing agreement from ARM Limited or its affiliates.
 *
 *        (C) COPYRIGHT 2008-2009, 2011-2017 ARM Limited or its affiliates.
 *             ALL RIGHTS RESERVED
 *
 * This entire notice must be reproduced on all copies of this file
 * and copies of this file may only be made by a person if such person is
 * permitted to do so under the terms of a subsisting license agreement
 * from ARM Limited or its affiliates.
 *
 * Release   : r0p0-00eac0
 *-----------------------------------------------------------------------------*/

/*
 *-----------------------------------------------------------------------------

 BRIEF DESCRIPTION, PURPOSE AND STRATEGY
 =======================================
 Uses a WriteValue job to write to a range of addresses in memory via the GPU and then reads back the written value via the CPU to confirm address lines are correctly wired up.

 DEBUG HINTS IN CASE OF FAILURE
 ===========================
 a) Check the AXI slave bus connections as this is the bus that is used to communicate with the Job manager.
 b) Check that the GPU's reset signal is not asserted.
 c) Check that the GPU input clock is toggling as expected.
 d) Check that the DFT signals are disabled.
 e) Check that MBIST is disabled.
 f) Check that each address and data line on the ACE interface is correctly wired up.
 g) Trace each read or write going to the external memory, making sure that there are no dropped or errored out transactions.

 *-----------------------------------------------------------------------------
 */


#include "../../src/MaliFns.h"
#include "../../src/MaliFns_DVALIN.h"
#include "../../src/MaliDefines_DVALIN.h"
#include "../../src/config.h"
#include "memory_DVALIN_check_0_a.h"
#include "memory_DVALIN_mem_0.h"

int RunMaliTest_memory_DVALIN_part0       (void);
int RunMaliTest_memory_DVALIN_part1       (void);

int Check_memory_DVALIN_check_0_a         (volatile struct t_memory_DVALIN_mem_0 *p);


static volatile struct t_memory_DVALIN_mem_0 *malidata_0;
static int mali_step;
static int max_address;
static int l2_slices;

int RunMaliTest_memory_DVALIN (int init) {
    if (init) { mali_step = 0; };
    switch (mali_step) {
        case 0  : return RunMaliTest_memory_DVALIN_part0();
        case 1  : return RunMaliTest_memory_DVALIN_part1();
        default : return 0;
    };
};

int matches_all_striping_granules (unsigned int addr)
{

    unsigned int hash_b0, hash_b1, l2_id;

    hash_b0 = (((addr>>8) & 0x1) ^ ((addr>>10) & 0x1) ^ ((addr>>13) & 0x1) ^ ((addr>>21) & 0x1));
    hash_b1 = (((addr>>9) & 0x1) ^ ((addr>>11) & 0x1) ^ ((addr>>16) & 0x1) ^ ((addr>>29) & 0x1));

    if (l2_slices == 4) l2_id = (hash_b1<<1) | hash_b0;
    else if  (l2_slices == 2) l2_id   = hash_b1 ^ hash_b0;
    else  l2_id   = 0;

#ifdef DEBUG_ON
    printf("hash_b1 : %x, hash_b0 : %x , ", hash_b1, hash_b0);
    printf("Addr : 0x%x, L2 is : %d\n", addr, l2_id);
#endif

    return l2_id;
}

int RunMaliTest_memory_DVALIN_part0 (void) {
    int res             = 0;
    int gpu_rawstat     = Mali_RdReg(GPU_IRQ_RAWSTAT);
    int gpu_status      = Mali_RdReg(GPU_IRQ_STATUS);
    int mmu_irq_rawstat = Mali_RdReg(MMU_IRQ_RAWSTAT);
    int num_cores, i, core_bitmap;
    volatile unsigned int * write_value_descriptor__field0;
#ifdef DEBUG_ON
    volatile unsigned int * write_value_descriptor__tmp;
#endif
    volatile unsigned int * last_job_descriptor__next_job_address;

    unsigned int current_job_slot, l2c, job_descriptor_counter;

    printf("RUNNING TEST: memory\n");
    printf("PURPOSE     : Tests correct access to the available system memory space\n");

    /* Get current configuration information */
    core_bitmap   = Mali_RdReg(0x0100);
    l2_slices     = ((Mali_RdReg(0x10)>>8)&0xf)+1;

    i = core_bitmap;
    num_cores = 0;
    while(i) {
        num_cores++;
        i &= i - 1;
    }

    Mali_Reset();

    /* ********************************** */
    printf ("Power on the shader cores\n");
    Mali_WrReg(SHADER_PWRON_LO     , 0xFFFFFFFF);
    printf ("Power on the Tiler\n");
    Mali_WrReg(TILER_PWRON_LO      , 0xFFFFFFFF);

    printf ("Poll the power changed all flag until it is set\n");
    (void) Mali_PollReg (GPU_IRQ_RAWSTAT, 10, 1, "GPU_IRQ_RAWSTAT[POWER_CHANGED_ALL]");
    gpu_rawstat = Mali_RdReg(GPU_IRQ_RAWSTAT);
    printf ("GPU IRQ RAWSTAT = 0x%x\n", gpu_rawstat);
    /* ********************************** */

    /* clear IRQs and set all masks */
    Mali_clear_irqs_and_set_all_masks();

#ifdef DEBUG_ON
    printf (" DBG: Check that the dummy descriptor got correctly loaded into memory\n");
#endif
    malidata_0 = Mali_LdMem((void *)&memory_DVALIN_mem_0,sizeof(struct t_memory_DVALIN_mem_0),(int)memory_DVALIN_mem_0.ttb);
    Check_memory_DVALIN_check_0_a (malidata_0);

#ifdef DEBUG_ON
    printf (" DBG: Setup the four MMU address spaces in Identity mode \n");
#endif
    Mali_write_ASn_register (ASn_TRANSTAB_LO, 0, 0x00000001);
    Mali_write_ASn_register (ASn_TRANSTAB_LO, 1, 0x00000001);
    Mali_write_ASn_register (ASn_TRANSTAB_LO, 2, 0x00000001);
    Mali_write_ASn_register (ASn_TRANSTAB_LO, 3, 0x00000001);

#ifdef DEBUG_ON
    printf (" DBG: Execute the update command for four MMU address spaces\n");
#endif
    Mali_write_ASn_register (ASn_COMMAND, 0, 0x00000001);
    Mali_write_ASn_register (ASn_COMMAND, 1, 0x00000001);
    Mali_write_ASn_register (ASn_COMMAND, 2, 0x00000001);
    Mali_write_ASn_register (ASn_COMMAND, 3, 0x00000001);

    mmu_irq_rawstat = Mali_RdReg(MMU_IRQ_RAWSTAT);
#ifdef DEBUG_ON
    printf (" DBG: mmu_irq_rawstat=0x%x\n", mmu_irq_rawstat);
#endif

    /* set the job slot to be used */
    current_job_slot = 0;

    max_address = (int)((char*)memory_DVALIN_mem_0.ttb+0x5000-1);//MAX_ADDRESS_VALUE;

    printf ("Configure job slot %d ready for the WriteValue job descriptors\n", current_job_slot);

    write_value_descriptor__field0 = (volatile unsigned int *)memory_DVALIN_mem_0.ttb;

    Mali_WrReg(JOB_IRQ_CLEAR       , 0xFFFFFFFF);
    Mali_WrReg(JOB_IRQ_MASK        , 0xFFFFFFFF);

    Mali_write_JSn_register(JSn_CONFIG_NEXT      , current_job_slot, 0x00003000); /* clean&invalidate caches at the end of a job */

    printf ("Prepare the WriteValue job descriptors\n");
    /*
       Word 8 is the WriteAddress - Lo Word

       Set the WriteAddress to four words along
       i.e.
       Word(s)  0-10 WriteValue Job descriptor
       Word(s)    11 Unused
       Word(s) 12-13 <64 bit value to be written>
       Word(s) 14-15 Unused
       */

    /* initialise a counter for the descriptors in the chain */
    job_descriptor_counter = 0;

    /* used to track the next job descriptor address in the chain */
    last_job_descriptor__next_job_address = 0;

    /* loop until max_address is reached */
    while ((unsigned int)write_value_descriptor__field0 < (max_address - 15))
    {
        l2c = matches_all_striping_granules((unsigned int ) (write_value_descriptor__field0+12));

        /* If this is the first descriptor then set its address in the selected job slot */
        if (job_descriptor_counter==0)
        {
            Mali_write_JSn_register(JSn_HEAD_NEXT_LO     , current_job_slot, (unsigned int)write_value_descriptor__field0);
        }

#ifdef DEBUG_ON
        printf ("Setting up WriteValue[%d] descriptor at 0x%x to write through l2c%d\n", job_descriptor_counter, write_value_descriptor__field0, l2c);
#endif

        /* increment the job_descriptor counter */
        job_descriptor_counter++;

#ifdef DEBUG_ON
        /* Save the descriptor address for debug purposes */
        write_value_descriptor__tmp = write_value_descriptor__field0;
#endif

        /* Set the next_job_address field of the last descriptor to the address of this one */
        if (last_job_descriptor__next_job_address)
        {
            *last_job_descriptor__next_job_address = (unsigned int) write_value_descriptor__field0;
#ifdef DEBUG_ON
            printf ("Setting up the next job pointer 0x%x for the last descriptor to this descriptor's address\n", last_job_descriptor__next_job_address);
#endif

        }

        /*
           write the header [ words 0-7 ]
           */
        /* zero the status words 0-3 */
        *write_value_descriptor__field0 = 0x00000000;
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;

        /*
           Words 4 and 5 - Flags and Indices

           0 -- Job descriptor size        -> 64 bit job descriptor
           7:1 -- Job type                   -> WriteValue
           8 -- Perform job barrier        -> No
           9 -- Perform cache invalidation
           on current address space   -> No
           10 -- Reserved
           11 -- Suppress Job Prefetch      -> Yes
           12 -- Texture Mapper enable      -> No
           13 -- Reserved
           14 -- Relax Dependency 1         -> 0
           15 -- Relax Dependency 2         -> 0
           31:16 -- Job Index                  -> Start at 0x1
        */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000805|(((job_descriptor_counter)&0xff)<<16);
        /*
        47:32 -- Job Dependency Index 1         -> 0
        63:48 -- Job Dependency Index 2         -> 0
        */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;
        /*
           Word 6 - pointer to the next job.
           */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;

        last_job_descriptor__next_job_address = write_value_descriptor__field0;


        /*
           Word 7 is reserved for a 32 bit job descriptor
           */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;

        /*
           Word 8 is the WriteAddress - Lo Word

           Set the WriteAddress to four words along
           i.e.
           Word(s)  0-10 WriteValue Job descriptor
           Word(s)    11 Unused
           Word(s) 12-13 <64 bit value to be written>
           Word(s) 14-15 Unused
           */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = (unsigned int) (write_value_descriptor__field0 + 0x4);

        /*
           Word 9 is the WriteAddress - Hi Word
           */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;

        /*
           Word 10 is the ValueDescriptor
           -- set to immediate64 i.e. 'h7
           */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000007;
        /*
           Word 11 is reserved
           --  skip
           */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;

        /*
           Word 12-13 are the immediate values to be written by the WriteValue job
           */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0x00000000;

        /*
           Skip words 14 and 15
           -> 14-15 are unused.
           -> Set to all 1s
           */
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0xffffffff;
        write_value_descriptor__field0++;
        *write_value_descriptor__field0 = 0xffffffff;


#ifdef DEBUG_ON
        {
            int j;
            for (j=0;j<16;j++)
            {
                if ((j%4)==0 && j != 0)
                {
                    printf ("\n  ");
                }
                printf ("word[%02d]=0x%08x ", j, *write_value_descriptor__tmp);
                write_value_descriptor__tmp += 1;
            }
            printf ("\n");
        }
#endif

        /* Advance the address for the descriptors by 1024 */
        write_value_descriptor__field0 += 241;
    }




    printf ("Now kick off the job chain\n");

    Mali_write_JSn_register(JSn_COMMAND_NEXT     , current_job_slot, 0x00000001);

    mali_step++;

    if (res==0)
    {
        return 255;
    }
    return res;
}

int RunMaliTest_memory_DVALIN_part1 (void) {
    int gpu_irq_status, job_irq_status, mmu_irq_status,event_irq_status;
    volatile unsigned int * write_value_descriptor__field0;
#ifdef DEBUG_ON
    volatile unsigned int * write_value_descriptor__tmp;
#endif

    unsigned int l2c, job_descriptor_counter;

    gpu_irq_status   = Mali_RdReg(GPU_IRQ_STATUS);
    job_irq_status   = Mali_RdReg(JOB_IRQ_STATUS);
    mmu_irq_status   = Mali_RdReg(MMU_IRQ_STATUS);
    event_irq_status = Mali_RdReg(EVENT_IRQ_STATUS);

    if ( (gpu_irq_status == 0) && ((job_irq_status == 1) || (job_irq_status == 2) || (job_irq_status == 4) || (job_irq_status == 8) ) && (mmu_irq_status == 0) && (event_irq_status == 0))
    {
        printf("JOB chain complete\n");
        Mali_WrReg(JOB_IRQ_CLEAR  , 0xFFFFFFFF); // JOB_IRQ_CLEAR
        Mali_WrReg(JOB_IRQ_MASK   , 0x00000000); // JOB_IRQ_MASK
        Mali_WrReg(MMU_IRQ_MASK   , 0x00000000); // MMU_IRQ_MASK
        Mali_WrReg(MMU_IRQ_RAWSTAT, 0x00000000); // MMU_IRQ_RAWSTAT
        Mali_WrReg(JOB_IRQ_RAWSTAT, 0x00000000); // JOB_IRQ_RAWSTAT
        mali_step++;

        /* Now read back the values and check that they are all zero */
        job_descriptor_counter = 0;

        write_value_descriptor__field0 = (volatile unsigned int *)memory_DVALIN_mem_0.ttb;
        while ((unsigned int)write_value_descriptor__field0 <= (max_address - 15))
        {
            l2c = matches_all_striping_granules((unsigned int ) (write_value_descriptor__field0+12));
            write_value_descriptor__field0 += 12;
#ifdef DEBUG_ON
            write_value_descriptor__tmp = write_value_descriptor__field0 - 12;
            printf ("Checking WriteValue[%d] descriptor at 0x%x which wrote zeros to 0x%x (the first two words) from l2c%d\n", job_descriptor_counter, (write_value_descriptor__field0 - 12), write_value_descriptor__field0, l2c);
#endif

#ifdef DEBUG_ON
            {
                int j;
                for (j=0;j<16;j++)
                {
                    if ((j%4)==0 && j != 0)
                    {
                        printf ("\n  ");
                    }
                    printf ("word[%02d]=0x%08x ", j, *write_value_descriptor__tmp);
                    write_value_descriptor__tmp += 1;
                }
                printf ("\n");
            }
#endif

            if (*write_value_descriptor__field0 == 0)
            {
                write_value_descriptor__field0 += 244;
                job_descriptor_counter++;
            }
            else
            {
                printf ("Found a none-zero value [0x%x] for WriteValue[%d] at 0x%x\n", *write_value_descriptor__field0, job_descriptor_counter, write_value_descriptor__field0);
                return 1;
            }
        }
        return 0;
    }

    printf("FAILURE: Expected JOB interrupt. Got the following interrupts: GPU: %x  JOB: %x  MMU: %x  EVENT: %x\n",
            gpu_irq_status, job_irq_status, mmu_irq_status, event_irq_status);
    Mali_WrReg(GPU_IRQ_CLEAR, 0xFFFFFFFF); // GPU_IRQ_CLEAR
    Mali_WrReg(JOB_IRQ_CLEAR, 0xFFFFFFFF); // JOB_IRQ_CLEAR
    Mali_WrReg(MMU_IRQ_CLEAR, 0xFFFFFFFF); // MMU_IRQ_CLEAR
    Mali_WrReg(EVENT_IRQ_CLEAR, 0xFFFFFFFF); // EVENT_IRQ_CLEAR
    Mali_WrReg(GPU_IRQ_MASK , 0x00000000); // GPU_IRQ_MASK
    Mali_WrReg(JOB_IRQ_MASK , 0x00000000); // JOB_IRQ_MASK
    Mali_WrReg(MMU_IRQ_MASK , 0x00000000); // MMU_IRQ_MASK
    Mali_WrReg(EVENT_IRQ_MASK , 0x00000000); // EVENT_IRQ_MASK
    return 1;
}

int Check_memory_DVALIN_check_0_a (volatile struct t_memory_DVALIN_mem_0 *p) {
    struct t_CheckData_memory_DVALIN_check_0_a *r = &CheckData_memory_DVALIN_check_0_a;
    int res = 0;
    Mali_Message("Performing data check of 64 bytes of the dummy descriptor\n");

    res |= Mali_MemCmp(p->data_00001000, 0x00001000, r->data_00001000, 0, 16);
    /* Return -1 if a compare fail */
    if (res) { return -1; };
    Mali_Message("Check okay \n");
    return 0;
};
