/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited or its affiliates.
*
*        (C) COPYRIGHT 2008-2009, 2011-2017 ARM Limited or its affiliates.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited or its affiliates.
*
* Release   : r0p0-00eac0
*-----------------------------------------------------------------------------*/
  
/* 

BRIEF DESCRIPTION, PURPOSE AND STRATEGY
=======================================
This test checks that the GPU's power domains can be turned on and off (SHADER cores, Tiler and L2C).

The various domains are confirmed to be on or off by performing a system wide cache flush and then checking that the GPU_ACTIVE flag is high or low.



DEBUG HINTS IN CASE OF FAILURE
===========================
a) Check the AXI bus connections as this is the bus that is used to communicate with the Job manager.
b) Check that the GPU's reset signal is not asserted.
c) Check that the GPU input clock is toggling as expected.
d) Check that the DFT signals are disabled.
e) Check that MBIST is disabled.

*/

#include "MaliFns.h"
#include "MaliDefines_DVALIN.h"


int RunMaliTest_pdc_access_DVALIN_part0 (void);

static int mali_step;

int RunMaliTest_pdc_access_DVALIN (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_pdc_access_DVALIN_part0();
    default : return 0;
  };
};


/* Mali_test_reg
   Test the register.
   access: 0=RW, 1=RO
*/
int Mali_test_reg(  int regnum, 
                    int read_mask, 
                    int write_mask, 
                    int check_value, 
                    int access,
                    char * reg_name) 
{
  int value = Mali_RdReg(regnum);
  if( value != check_value ) 
  {
    printf("FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x  for  %s\n",
           regnum, value, check_value, reg_name);
    return -1;
  }
  if( access == 0 ) 
  {
    Mali_WrReg(regnum, (0xffffffff & write_mask));
    value = Mali_RdReg(regnum) & read_mask;
    if( value != (0xffffffff & write_mask & read_mask) ) 
    {
      printf("FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x  for  %s\n",
             regnum, value, (0xffffffff & write_mask & read_mask), reg_name);
      return -2;
    }
    Mali_WrReg(regnum, (0x12345678 & write_mask));
    value = Mali_RdReg(regnum) & read_mask;
    if( value != (0x12345678 & write_mask & read_mask) ) 
    {
      printf("FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x  for  %s\n",
             regnum, value, (0x12345678 & write_mask & read_mask), reg_name);
      return -2;
    }
  }
  printf("Register %08x: Success!  for  %s\n", regnum, reg_name);
  return 0;
}


//=================================================================
int GPU_activity()
{
  int gpu_status = Mali_RdReg(GPU_IRQ_STATUS);
  /* ********************************** */
  printf ("Starting GPU activity \n");
  gpu_status = Mali_RdReg(GPU_STATUS);
  printf ("GPU_STATUS = %x\n", gpu_status);
  printf ("Starting flush operation....\n");
  Mali_WrReg(GPU_COMMAND, 8); 
  gpu_status = Mali_RdReg(GPU_STATUS);
  printf ("GPU_STATUS = %x\n", gpu_status);
  /* ********************************** */
  return gpu_status;
  

}

int RunMaliTest_pdc_access_DVALIN_part0 (void) {
  int res = 0;

  int num_cores, i, core_bitmap;
  int active;

  printf("RUNNING TEST: pdc_access\n");
  printf("  Purpose: Test correct integration of power domain controllers\n");
  printf("           Each power domain is accessed to ensure the core is actually powered up & down\n");

  // Get current configuration to allow testing all registers
  core_bitmap   = Mali_RdReg(0x0100);
  
  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i &= i-1;
  }

  Mali_Reset();

  res |= Mali_test_reg(0x000, 0xffff0000, 0xffff0000, GPU_ID_VALUE, 1, "GPU_ID");           
  res |= Mali_test_reg(0x100, 0xffffffff, 0xffffffff, core_bitmap,1, "SHADER_PRESENT_LO");
  res |= Mali_test_reg(0x110, 0xffffffff, 0xffffffff, 0x00000001, 1, "TILER_PRESENT_LO"); 
  res |= Mali_test_reg(0x120, 0xffffffff, 0xffffffff, 0x00000001, 1, "L2_PRESENT_LO");    
  res |= Mali_test_reg(0x140, 0xffffffff, 0xffffffff, 0x00000000, 1, "SHADER_READY_LO");  
  res |= Mali_test_reg(0x150, 0xffffffff, 0xffffffff, 0x00000000, 1, "TILER_READY_LO");   
  res |= Mali_test_reg(0x160, 0xffffffff, 0xffffffff, 0x00000000, 1, "L2_READY_LO");      

  // --------------------------------------------------------
  printf("POWERING OFF \n");  
  for(i=0; i<num_cores; i++) 
  {
    printf("Power OFF Shader Core #%d\n", i);
    Mali_WrReg(SHADER_PWROFF_LO, (1<<i)); 
    while( Mali_RdReg(SHADER_PWRTRANS_LO) ) {}
  }

  printf("Power OFF Tiler\n");
  Mali_WrReg(TILER_PWROFF_LO, 1); 
  while( Mali_RdReg(0x210) ) {}
  res |= Mali_test_reg(0x150, 0xffffffff, 0xffffffff, 0, 1, "TILER_READY_LO"); 

  printf("Power OFF L2\n");
  Mali_WrReg(L2_PWROFF_LO, 1); 
  while( Mali_RdReg(0x220) ) {}
  res |= Mali_test_reg(0x160, 0xffffffff, 0xffffffff, 0, 1, "L2_READY_LO");

  active = GPU_activity();
  if (active)
  {
     printf("FAILURE: GPU Active when Powered OFF!\n");
  }
  else
  {
     printf("SUCCESS: GPU Inactive when Powered OFF\n");
  }

  // --------------------------------------------------------
  printf("POWERING ON \n");
  for(i=0; i<num_cores; i++) 
  {   
    printf("Power ON  Shader Core #%d\n", i);
    Mali_WrReg(SHADER_PWRON_LO, (1<<i)); 
    while( Mali_RdReg(SHADER_PWRTRANS_LO) ) {}
  }
  
  printf("Power ON  Tiler\n");
  Mali_WrReg(TILER_PWRON_LO, 1); 
  while( Mali_RdReg(0x210) ) {}
  res |= Mali_test_reg(0x150, 0xffffffff, 0xffffffff, 1, 1, "TILER_READY_LO"); 

  printf("Power ON  L2\n");
  Mali_WrReg(L2_PWRON_LO, 1); 
  while( Mali_RdReg(0x220) ) {}
  res |= Mali_test_reg(0x160, 0xffffffff, 0xffffffff, 1, 1, "L2_READY_LO");
  active = GPU_activity();
  if (!active)
  {
     printf("FAILURE: GPU Inactive when Powered ON!\n");
  }
  else
  {
     printf("SUCCESS: GPU Active when Powered ON\n");
  }



  // --------------------------------------------------------
  printf("POWERING OFF \n");  
  for(i=0; i<num_cores; i++) 
  {
    printf("Power OFF Shader Core #%d\n", i);
    Mali_WrReg(SHADER_PWROFF_LO, (1<<i)); 
    while( Mali_RdReg(SHADER_PWRTRANS_LO) ) {}
  }

  printf("Power OFF Tiler\n");
  Mali_WrReg(TILER_PWROFF_LO, 1); 
  while( Mali_RdReg(0x210) ) {}
  res |= Mali_test_reg(0x150, 0xffffffff, 0xffffffff, 0, 1, "TILER_READY_LO"); /* TILER_READY_LO */

  printf("Power ON  L2\n");
  Mali_WrReg(L2_PWRON_LO, 1); 
  while( Mali_RdReg(0x220) ) {}
  res |= Mali_test_reg(0x160, 0xffffffff, 0xffffffff, 1, 1, "L2_READY_LO");
  printf("Power OFF L2\n");
  Mali_WrReg(L2_PWROFF_LO, 1); 
  while( Mali_RdReg(0x220) ) {}
  res |= Mali_test_reg(0x160, 0xffffffff, 0xffffffff, 0, 1, "L2_READY_LO");

  active = GPU_activity();
  if (active)
  {
     printf("FAILURE: GPU Active when Powered OFF!\n");
  }
  else
  {
     printf("SUCCESS: GPU Inactive when Powered OFF\n");
  }


  // --------------------------------------------------------

  mali_step++;
  return res;
}
