/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited or its affiliates.
*
*        (C) COPYRIGHT 2008-2009, 2011-2017 ARM Limited or its affiliates.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited or its affiliates.
*
* Release   : r0p0-00eac0
*-----------------------------------------------------------------------------*/

/* 

BRIEF DESCRIPTION, PURPOSE AND STRATEGY
=======================================

This test checks that the GPU's different power domains can be turned on and off.

It makes sure the power on or off status corresponds to the relevant domain's READY status.


DEBUG HINTS IN CASE OF FAILURE
===========================
a) Check the AXI bus connections as this is the bus that is used to communicate with the Job manager.
b) Check that the GPU's reset signal is not asserted.
c) Check that the GPU input clock is toggling as expected.
d) Check that the DFT signals are disabled.
e) Check that MBIST is disabled.

*/

#include "MaliFns.h"
#include "MaliDefines_DVALIN.h"

int RunMaliTest_pdc_on_off_DVALIN_part0 (void);

static int mali_step;

int RunMaliTest_pdc_on_off_DVALIN (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_pdc_on_off_DVALIN_part0();
    default : return 0;
  };
};

/* Mali_test_reg
   Test the register.
   access: 0=RW, 1=RO
*/
int Mali_test_reg(int regnum, int read_mask, int write_mask, int reset_value, int access) {
  int value = Mali_RdReg(regnum);
  if( value != reset_value ) {
    printf("FAILURE: Wrong reset value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x\n",
           regnum, value, reset_value);
    return -1;
  }
  if( access == 0 ) {
    Mali_WrReg(regnum, (0xffffffff & write_mask));
    value = Mali_RdReg(regnum) & read_mask;
    if( value != (0xffffffff & write_mask & read_mask) ) {
      printf("FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x\n",
             regnum, value, (0xffffffff & write_mask & read_mask));
      return -2;
    }
    Mali_WrReg(regnum, (0x12345678 & write_mask));
    value = Mali_RdReg(regnum) & read_mask;
    if( value != (0x12345678 & write_mask & read_mask) ) {
      printf("FAILURE: Wrong value. Addr: 0x%08x Value: 0x%08x Expected: 0x%08x\n",
             regnum, value, (0x12345678 & write_mask & read_mask));
      return -2;
    }
  }
  printf("Register %08x: Success!\n", regnum);
  return 0;
}

int RunMaliTest_pdc_on_off_DVALIN_part0 (void) {
  int res = 0;

  int num_cores, i, j, skip_core;
  volatile unsigned int core_bitmap;

  printf("RUNNING TEST: pdc_on_off\n");
  printf("  Purpose: To check the correct integration of the power domain controller adapters.\n");
  printf("  This test powers up and powers down each power domain.\n");

  // Get current configuration to allow testing all registers
  core_bitmap   = Mali_RdReg(0x0100);

  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i &= i-1;
  }
  
  res |= Mali_test_reg(0x000, 0xffff0000, 0xffff0000, GPU_ID_VALUE, 1); /* GPU_ID */
  res |= Mali_test_reg(0x100, 0xffffffff, 0xffffffff,  core_bitmap, 1); /* SHADER_PRESENT_LO */
  res |= Mali_test_reg(0x110, 0xffffffff, 0xffffffff, 0x00000001, 1); /* TILER_PRESENT_LO */
  res |= Mali_test_reg(0x120, 0xffffffff, 0xffffffff, 0x00000001, 1); /* L2_PRESENT_LO */
  res |= Mali_test_reg(0x140, 0xffffffff, 0xffffffff, 0x00000000, 1); /* SHADER_READY_LO */
  res |= Mali_test_reg(0x150, 0xffffffff, 0xffffffff, 0x00000000, 1); /* TILER_READY_LO */
  res |= Mali_test_reg(0x160, 0xffffffff, 0xffffffff, 0x00000000, 1); /* L2_READY_LO */

  for(i=0; core_bitmap!=0; i++) {
    
    if (core_bitmap & 0x1) {
        skip_core = 0;
    } else {
        skip_core = 1;
    }

    core_bitmap = core_bitmap >> 1;
    j = ((i + num_cores) % 4);

    if (skip_core) {
      printf("Skipping shader core #%d\n", i);
    } else {
      printf("Power on/off shader core #%d\n", i);
      Mali_WrReg(0x180, (1<<i)); /* SHADER_PWRON_LO */
      while( Mali_RdReg(0x200) ) {}
      res |= Mali_test_reg(0x140, 0xffffffff, 0xffffffff, (1<<i), 1); /* SHADER_READY_LO */

      printf("Powering off shader core #%d\n", i);
      Mali_WrReg(0x1c0, (1<<i)); /* SHADER_PWROFF_LO */

      while( Mali_RdReg(0x200) ) {}
      res |= Mali_test_reg(0x140, 0xffffffff, 0xffffffff, 0, 1); /* SHADER_READY_LO */
    }
  }
  printf("Power on/off tiler\n");
  Mali_WrReg(0x190, 1); /* TILER_PWRON_LO */
  while( Mali_RdReg(0x210) ) {}
  res |= Mali_test_reg(0x150, 0xffffffff, 0xffffffff, 1, 1); /* TILER_READY_LO */
  Mali_WrReg(0x1d0, 1); /* TILER_PWROFF_LO */
  while( Mali_RdReg(0x210) ) {}
  res |= Mali_test_reg(0x150, 0xffffffff, 0xffffffff, 0, 1); /* TILER_READY_LO */

  printf("Power on/off L2\n");
  Mali_WrReg(0x1a0, 1); /* L2_PWRON_LO */
  while( Mali_RdReg(0x220) ) {}
  res |= Mali_test_reg(0x160, 0xffffffff, 0xffffffff, 1, 1); /* L2_READY_LO */
  Mali_WrReg(0x1e0, 1); /* L2_PWROFF_LO */
  while( Mali_RdReg(0x220) ) {}
  res |= Mali_test_reg(0x160, 0xffffffff, 0xffffffff, 0, 1); /* L2_READY_LO */
  mali_step++;
  return res;
}
